/*
 * AuthorSlaveAuthorGroupPanel.java
 *
 * Created on 21-mar-2011, 11:34:59
 */
package scimat.gui.components.slavepanel;

import java.util.ArrayList;
import scimat.gui.components.ErrorDialogManager;
import scimat.model.knowledgebase.entity.Author;
import scimat.model.knowledgebase.entity.AuthorGroup;
import scimat.model.knowledgebase.exception.KnowledgeBaseException;
import scimat.project.CurrentProject;
import scimat.project.observer.AuthorGroupRelationAuthorObserver;
import scimat.project.observer.EntityObserver;

/**
 *
 * @author mjcobo
 */
public class AuthorSlaveAuthorGroupPanel extends GenericOneSlaveItemPanel
        implements AuthorGroupRelationAuthorObserver, EntityObserver<AuthorGroup> {

  /** Creates new form WordSlaveWordGroupPanel */
  public AuthorSlaveAuthorGroupPanel() {
    initComponents();

    CurrentProject.getInstance().getKbObserver().addAuthorGroupRelationAuthorsObserver(this);
    CurrentProject.getInstance().getKbObserver().addAuthorGroupObserver(this);
  }

  /**
   *
   */
  private void refresh() {

    if (this.authorGroup != null) {

      this.groupNameTextField.setText(this.authorGroup.getGroupName());
      this.stopGroupCheckBox.setSelected(this.authorGroup.isStopGroup());
      fireSlaveItemObserver(true);

    } else {

      this.groupNameTextField.setText("");
      this.stopGroupCheckBox.setSelected(false);
    }
  }

  /**
   *
   */
  public void setMasterItem(Author author) {

    this.author = author;

    try {

      if (this.author != null) {

        relationChanged();

      } else {

        this.authorGroup = null;
        refresh();

      }

    } catch (KnowledgeBaseException e) {
    
      ErrorDialogManager.getInstance().showException(e);

    }
  }

  /**
   *
   */
  public void relationChanged() throws KnowledgeBaseException {

    if (this.author != null) {
    
      this.authorGroup = CurrentProject.getInstance().getFactoryDAO().getAuthorDAO().getAuthorGroup(this.author.getAuthorID());
      
    } else {
    
      this.authorGroup = null;
      
    }
    
    refresh();
  }

  /**
   *
   * @param items
   * @throws KnowledgeBaseException
   */
  public void entityAdded(ArrayList<AuthorGroup> items) throws KnowledgeBaseException {
    // Do not do nothing
  }

  /**
   *
   * @param entity
   * @throws KnowledgeBaseException
   */
  public void entityRefresh() throws KnowledgeBaseException {

    if (this.authorGroup != null) {

      this.authorGroup = CurrentProject.getInstance().getFactoryDAO().getAuthorGroupDAO().getAuthorGroup(this.authorGroup.getAuthorGroupID());
      refresh();
    }
  }

  /**
   *
   * @param items
   * @throws KnowledgeBaseException
   */
  public void entityRemoved(ArrayList<AuthorGroup> items) throws KnowledgeBaseException {
    // Do not do nothing
  }

  /**
   *
   * @param items
   * @throws KnowledgeBaseException
   */
  public void entityUpdated(ArrayList<AuthorGroup> items) throws KnowledgeBaseException {

    int position;

    if (this.authorGroup != null) {

      position = items.indexOf(this.authorGroup);

      if (position != -1) {

        this.authorGroup = items.get(position);
      }
    }
  }

  /** This method is called from within the constructor to
   * initialize the form.
   * WARNING: Do NOT modify this code. The content of this method is
   * always regenerated by the Form Editor.
   */
  @SuppressWarnings("unchecked")
  // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
  private void initComponents() {

    groupNameDescriptionLabel = new javax.swing.JLabel();
    groupNameTextField = new javax.swing.JTextField();
    stopGroupCheckBox = new javax.swing.JCheckBox();

    groupNameDescriptionLabel.setText("Group name:");

    groupNameTextField.setEditable(false);

    stopGroupCheckBox.setText("is stop group?");
    stopGroupCheckBox.setEnabled(false);

    javax.swing.GroupLayout layout = new javax.swing.GroupLayout(this);
    this.setLayout(layout);
    layout.setHorizontalGroup(
      layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(layout.createSequentialGroup()
        .addComponent(groupNameDescriptionLabel)
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(groupNameTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 245, Short.MAX_VALUE))
      .addGroup(layout.createSequentialGroup()
        .addComponent(stopGroupCheckBox)
        .addContainerGap())
    );
    layout.setVerticalGroup(
      layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
      .addGroup(layout.createSequentialGroup()
        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
          .addComponent(groupNameDescriptionLabel)
          .addComponent(groupNameTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
        .addComponent(stopGroupCheckBox))
    );
  }// </editor-fold>//GEN-END:initComponents
  // Variables declaration - do not modify//GEN-BEGIN:variables
  private javax.swing.JLabel groupNameDescriptionLabel;
  private javax.swing.JTextField groupNameTextField;
  private javax.swing.JCheckBox stopGroupCheckBox;
  // End of variables declaration//GEN-END:variables
  private Author author = null;
  private AuthorGroup authorGroup = null;
}
